"""Module for the UI of the ExportPropertiesDialog."""
# Form implementation generated from reading ui file 'Window_exportProperties_Nov24.ui'
#
# Created by: PyQt6 UI code generator 6.4.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets

# ## Name to function mapping
# input
# spinBox_5 = voxel_size
# spinBox_6 = background
# spinBox_8 = feret_angle
# spinBox_7 = mesh_spacing
# spinBox_9 = start_slice
# spinBox_10 = end_slice
# spinBox_3 = remove_small
# spinBox_4 = num_threads

# checkBox_24 = save_labels
# checkBox_5 = inclusions

# output
# checkBox_34 = inertia
# checkBox_35 = ferets
# checkBox_27 = entropy
# checkBox_29 = mean_max
# checkBox_21 = aspect_ratio
# checkBox_33 = moments
# checkBox_36 = euler
# checkBox_31 = solidity

# checkBox_37 = histograms
# checkBox_38 = basic_properties
# checkBox_23 = select_all

# buttons
# pushButton = subdataset
# pushButton_2 = all_data
# pushButton_3 = cancel


class ExportPropertiesDialog:
    """Class for the Export Properties Dialog UI."""

    def setupUi(self, Dialog):
        """Set up the UI for the Export Properties Dialog."""
        Dialog.setObjectName("Dialog")
        Dialog.resize(485, 284)
        self.verticalLayoutWidget_2 = QtWidgets.QWidget(parent=Dialog)
        self.verticalLayoutWidget_2.setGeometry(QtCore.QRect(10, 10, 471, 275))
        self.verticalLayoutWidget_2.setObjectName("verticalLayoutWidget_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_2)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.verticalLayout_12 = QtWidgets.QVBoxLayout()
        self.verticalLayout_12.setObjectName("verticalLayout_12")
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label_9 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_9.setToolTip("")
        self.label_9.setWhatsThis("")
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_12.addWidget(self.label_9)
        self.spinBox_5 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_5.setMouseTracking(False)
        self.spinBox_5.setToolTip("")
        self.spinBox_5.setToolTipDuration(20)
        self.spinBox_5.setStatusTip("")
        self.spinBox_5.setWhatsThis("")
        self.spinBox_5.setMaximum(1000)
        self.spinBox_5.setProperty("value", 5)
        self.spinBox_5.setObjectName("spinBox_5")
        self.horizontalLayout_12.addWidget(self.spinBox_5)
        self.verticalLayout_12.addLayout(self.horizontalLayout_12)
        self.verticalLayout_7.addLayout(self.verticalLayout_12)
        self.verticalLayout_13 = QtWidgets.QVBoxLayout()
        self.verticalLayout_13.setObjectName("verticalLayout_13")
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_10 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_13.addWidget(self.label_10)
        self.spinBox_6 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_6.setMaximum(30000)
        self.spinBox_6.setSingleStep(100)
        self.spinBox_6.setProperty("value", 500)
        self.spinBox_6.setObjectName("spinBox_6")
        self.horizontalLayout_13.addWidget(self.spinBox_6)
        self.verticalLayout_13.addLayout(self.horizontalLayout_13)
        self.verticalLayout_7.addLayout(self.verticalLayout_13)
        self.verticalLayout_15 = QtWidgets.QVBoxLayout()
        self.verticalLayout_15.setObjectName("verticalLayout_15")
        self.horizontalLayout_15 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_15.setObjectName("horizontalLayout_15")
        self.label_12 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout_15.addWidget(self.label_12)
        self.spinBox_8 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_8.setMaximum(60)
        self.spinBox_8.setSingleStep(2)
        self.spinBox_8.setProperty("value", 10)
        self.spinBox_8.setObjectName("spinBox_8")
        self.horizontalLayout_15.addWidget(self.spinBox_8)
        self.verticalLayout_15.addLayout(self.horizontalLayout_15)
        self.verticalLayout_7.addLayout(self.verticalLayout_15)
        self.verticalLayout_14 = QtWidgets.QVBoxLayout()
        self.verticalLayout_14.setObjectName("verticalLayout_14")
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label_11 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_14.addWidget(self.label_11)
        self.spinBox_7 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_7.setMaximum(10)
        self.spinBox_7.setProperty("value", 1)
        self.spinBox_7.setObjectName("spinBox_7")
        self.horizontalLayout_14.addWidget(self.spinBox_7)
        self.verticalLayout_14.addLayout(self.horizontalLayout_14)
        self.verticalLayout_7.addLayout(self.verticalLayout_14)
        self.verticalLayout.addLayout(self.verticalLayout_7)
        self.horizontalLayout.addLayout(self.verticalLayout)
        spacerItem = QtWidgets.QSpacerItem(
            15,
            20,
            QtWidgets.QSizePolicy.Policy.Fixed,
            QtWidgets.QSizePolicy.Policy.Minimum,
        )
        self.horizontalLayout.addItem(spacerItem)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_18 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_18.setObjectName("horizontalLayout_18")
        self.horizontalLayout_17 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_17.setObjectName("horizontalLayout_17")
        self.verticalLayout_17 = QtWidgets.QVBoxLayout()
        self.verticalLayout_17.setObjectName("verticalLayout_17")
        self.label_14 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_14.setToolTip("")
        self.label_14.setWhatsThis("")
        self.label_14.setObjectName("label_14")
        self.verticalLayout_17.addWidget(self.label_14)
        self.label_13 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_13.setToolTip("")
        self.label_13.setWhatsThis("")
        self.label_13.setObjectName("label_13")
        self.verticalLayout_17.addWidget(self.label_13)
        self.horizontalLayout_17.addLayout(self.verticalLayout_17)
        self.horizontalLayout_18.addLayout(self.horizontalLayout_17)
        self.horizontalLayout_16 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_16.setObjectName("horizontalLayout_16")
        self.verticalLayout_16 = QtWidgets.QVBoxLayout()
        self.verticalLayout_16.setObjectName("verticalLayout_16")
        self.spinBox_9 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_9.setMaximum(10000)
        self.spinBox_9.setObjectName("spinBox_9")
        self.spinBox_9.setMinimum(-1)
        self.spinBox_9.setProperty("value", -1)
        self.verticalLayout_16.addWidget(self.spinBox_9)
        self.spinBox_10 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_10.setMaximum(10000)
        self.spinBox_10.setObjectName("spinBox_10")
        self.spinBox_10.setMinimum(-1)
        self.spinBox_10.setProperty("value", -1)
        self.verticalLayout_16.addWidget(self.spinBox_10)
        self.horizontalLayout_16.addLayout(self.verticalLayout_16)
        self.horizontalLayout_18.addLayout(self.horizontalLayout_16)
        self.verticalLayout_4.addLayout(self.horizontalLayout_18)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.checkBox_24 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_24.setObjectName("checkBox_24")
        self.horizontalLayout_3.addWidget(self.checkBox_24)
        # self.checkBox_5 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        # self.checkBox_5.setObjectName("checkBox_5")
        # self.horizontalLayout_3.addWidget(self.checkBox_5)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_6 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_8.addWidget(self.label_6)
        self.spinBox_3 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_3.setMaximum(10000)
        self.spinBox_3.setProperty("value", 1000)
        self.spinBox_3.setObjectName("spinBox_3")
        self.horizontalLayout_8.addWidget(self.spinBox_3)
        self.verticalLayout_4.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_7 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_9.addWidget(self.label_7)
        self.spinBox_4 = QtWidgets.QSpinBox(parent=self.verticalLayoutWidget_2)
        self.spinBox_4.setMinimum(-1)
        self.spinBox_4.setMaximum(50)
        self.spinBox_4.setProperty("value", -1)
        self.spinBox_4.setObjectName("spinBox_4")
        self.horizontalLayout_9.addWidget(self.spinBox_4)
        self.verticalLayout_4.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_7.addLayout(self.verticalLayout_4)
        self.horizontalLayout.addLayout(self.horizontalLayout_7)
        spacerItem1 = QtWidgets.QSpacerItem(
            15,
            20,
            QtWidgets.QSizePolicy.Policy.Fixed,
            QtWidgets.QSizePolicy.Policy.Minimum,
        )
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        # self.pushButton = QtWidgets.QPushButton(parent=self.verticalLayoutWidget_2)
        # self.pushButton.setObjectName("pushButton")
        # self.horizontalLayout_4.addWidget(self.pushButton)
        self.horizontalLayout_6.addLayout(self.horizontalLayout_4)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.pushButton_2 = QtWidgets.QPushButton(parent=self.verticalLayoutWidget_2)
        self.pushButton_2.setObjectName("pushButton_2")
        self.verticalLayout_2.addWidget(self.pushButton_2)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.pushButton_3 = QtWidgets.QPushButton(parent=self.verticalLayoutWidget_2)
        self.pushButton_3.setObjectName("pushButton_3")
        self.horizontalLayout_5.addWidget(self.pushButton_3)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.verticalLayout_5.addLayout(self.verticalLayout_2)
        self.horizontalLayout.addLayout(self.verticalLayout_5)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.line_3 = QtWidgets.QFrame(parent=self.verticalLayoutWidget_2)
        self.line_3.setFrameShape(QtWidgets.QFrame.Shape.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Shadow.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout_3.addWidget(self.line_3)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout()
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.label_8 = QtWidgets.QLabel(parent=self.verticalLayoutWidget_2)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_8.setFont(font)
        self.label_8.setObjectName("label_8")
        self.verticalLayout_10.addWidget(self.label_8)
        self.checkBox_34 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_34.setObjectName("checkBox_34")
        self.verticalLayout_10.addWidget(self.checkBox_34)
        self.checkBox_35 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_35.setObjectName("checkBox_35")
        self.verticalLayout_10.addWidget(self.checkBox_35)
        self.checkBox_27 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_27.setObjectName("checkBox_27")
        self.verticalLayout_10.addWidget(self.checkBox_27)
        self.checkBox_29 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_29.setObjectName("checkBox_29")
        self.verticalLayout_10.addWidget(self.checkBox_29)
        self.horizontalLayout_2.addLayout(self.verticalLayout_10)
        self.verticalLayout_20 = QtWidgets.QVBoxLayout()
        self.verticalLayout_20.setObjectName("verticalLayout_20")
        self.checkBox_21 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_21.setObjectName("checkBox_21")
        self.verticalLayout_20.addWidget(self.checkBox_21)
        self.checkBox_33 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_33.setObjectName("checkBox_33")
        self.verticalLayout_20.addWidget(self.checkBox_33)
        self.checkBox_36 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_36.setObjectName("checkBox_36")
        self.verticalLayout_20.addWidget(self.checkBox_36)
        self.checkBox_31 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        self.checkBox_31.setObjectName("checkBox_31")
        self.verticalLayout_20.addWidget(self.checkBox_31)
        self.horizontalLayout_2.addLayout(self.verticalLayout_20)
        self.horizontalLayout_19 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_19.setObjectName("horizontalLayout_19")
        self.verticalLayout_19 = QtWidgets.QVBoxLayout()
        self.verticalLayout_19.setObjectName("verticalLayout_19")
        self.checkBox_37 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.checkBox_37.setFont(font)
        self.checkBox_37.setChecked(True)
        self.checkBox_37.setObjectName("checkBox_37")
        self.verticalLayout_19.addWidget(self.checkBox_37)
        self.checkBox_38 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.checkBox_38.setFont(font)
        self.checkBox_38.setChecked(True)
        self.checkBox_38.setObjectName("checkBox_38")
        self.verticalLayout_19.addWidget(self.checkBox_38)
        self.checkBox_23 = QtWidgets.QCheckBox(parent=self.verticalLayoutWidget_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.checkBox_23.setFont(font)
        self.checkBox_23.setObjectName("checkBox_23")
        self.verticalLayout_19.addWidget(self.checkBox_23)
        self.horizontalLayout_19.addLayout(self.verticalLayout_19)
        self.horizontalLayout_2.addLayout(self.horizontalLayout_19)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        """Set the text for the UI elements."""
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.label.setText(_translate("Dialog", "Inputs"))
        self.label_9.setText(_translate("Dialog", "Voxel Size (um)"))
        self.label_10.setText(_translate("Dialog", "Background"))
        self.label_12.setText(_translate("Dialog", "Feret angle"))
        self.label_11.setText(_translate("Dialog", "mesh spacing"))
        self.label_14.setText(_translate("Dialog", "Start slice"))
        self.label_13.setText(_translate("Dialog", "End slice"))
        self.checkBox_24.setText(_translate("Dialog", "Save labels"))
        # self.checkBox_5.setText(_translate("Dialog", "Inclusions"))
        self.label_6.setText(_translate("Dialog", "Remove Small (vx)"))
        self.label_7.setText(_translate("Dialog", "#Treads"))
        self.pushButton_2.setText(_translate("Dialog", "extract"))
        self.pushButton_3.setText(_translate("Dialog", "Cancel"))
        self.label_8.setText(_translate("Dialog", "Outputs"))
        self.checkBox_34.setText(_translate("Dialog", "inertia"))
        self.checkBox_35.setText(_translate("Dialog", "Ferets"))
        self.checkBox_27.setText(_translate("Dialog", "Entropy"))
        self.checkBox_29.setText(_translate("Dialog", "Mean/Max"))
        self.checkBox_21.setText(_translate("Dialog", "Aspect ratio"))
        self.checkBox_33.setText(_translate("Dialog", "Moments"))
        self.checkBox_36.setText(_translate("Dialog", "Euler"))
        self.checkBox_31.setText(_translate("Dialog", "Solidity"))
        self.checkBox_37.setText(_translate("Dialog", "Histograms"))
        self.checkBox_38.setText(_translate("Dialog", "Basic properties"))
        self.checkBox_23.setText(_translate("Dialog", "Select all"))


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    Dialog = QtWidgets.QDialog()
    ui = ExportPropertiesDialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec())



"""Controller for managing the streamlit data viewer dialog in the ari3d application."""
import os
from datetime import datetime
from pathlib import Path

from PyQt6 import QtWidgets
from PyQt6.QtCore import QThreadPool
from PyQt6.QtWidgets import QDialog, QMessageBox, QWidget

from ari3d.gui.ari3d_logging import Ari3dLogger
from ari3d.gui.controller.io.album_c import AlbumController
from ari3d.gui.controller.io.project_c import OpenProjectController
from ari3d.gui.model.tasks import RunSolutionTask
from ari3d.gui.view.solutions.streamlit_ui import StreamlitDialog


class DataViewerDialogController:
    """Controller for managing the streamlit data viewer dialog in the ari3d application."""

    def __init__(
        self,
        parent: QWidget,
        album_api: AlbumController,
        open_project_dialog_controller: OpenProjectController,
        interactive: bool = True,
    ):
        """Initialize the DataViewerDialogController with the parent widget, album API, and project controller."""
        self.parent = parent
        self.album_api = album_api
        self.project_controller = open_project_dialog_controller
        self.streamlit_window = QDialog()
        self.streamlit_window_ui = StreamlitDialog()
        self.logger = Ari3dLogger()
        self.interactive = interactive
        self.task = None

    def setup_ui(self):
        """Set up the UI for the streamlit data viewer dialog."""
        # Clear the existing layout if it exists
        if self.streamlit_window.layout() is not None:
            QtWidgets.QWidget().setLayout(self.streamlit_window.layout())

        self.streamlit_window_ui.setupUi(self.streamlit_window)
        self.streamlit_window.setWindowTitle("Streamlit Parameters")
        self.set_defaults()
        self.set_connections()

    @property
    def histogram_file(self):
        """Return the path to the histogram file."""
        return self._histogram_file

    def handle_missing_h5ad(self):
        """Handle the case where no histogram files are found."""
        self.logger.log.error(
            "No histogram files found, please ensure the analysis path is correct and try again."
        )
        if self.interactive:
            # show info message
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Icon.Information)
            msg.setText(
                "No histogram files found, please ensure the analysis path is correct and try again."
            )
            msg.setWindowTitle("Info")
            msg.exec()
            self.streamlit_window_ui.buttonBox.button(
                QtWidgets.QDialogButtonBox.StandardButton.Ok
            ).setEnabled(False)

    def load_data_path(self):
        """Load the data path for the streamlit solution."""
        path = QtWidgets.QFileDialog.getExistingDirectory(
            None,
            "Select Data Path",
            directory=str(self.project_controller.analysis_path),
        )

        if not path:
            return

        files = [f for f in os.listdir(path) if f.endswith(".h5ad")]
        if files:
            if self.interactive:
                self.streamlit_window_ui.data_pathTextEdit.setText(path)
                self.streamlit_window_ui.buttonBox.button(
                    QtWidgets.QDialogButtonBox.StandardButton.Ok
                ).setEnabled(True)
        else:
            self.handle_missing_h5ad()

    def load_report_path(self):
        """Load the report path for the streamlit solution."""
        path = QtWidgets.QFileDialog.getExistingDirectory(
            None,
            "Select Report Path",
            directory=str(self.project_controller.report_path),
        )

        if not path:
            return

        if self.interactive:
            self.streamlit_window_ui.report_pathTextEdit.setText(path)

    def load_parameters_file(self):
        """Load the parameters file for the streamlit solution."""
        path = QtWidgets.QFileDialog.getOpenFileName(
            None,
            "Select Parameters File",
            directory=str(self.project_controller.report_path),
            filter="YML Files (*.yml)",
        )[0]

        if not path:
            return

        if self.interactive:
            self.streamlit_window_ui.parameters_ymlTextEdit.setText(path)

    def set_defaults(self):
        """Set default paths for the streamlit solution dialog."""
        analysis_path = str(self.project_controller.analysis_path)

        if self.interactive:
            self.streamlit_window_ui.data_pathTextEdit.setText(analysis_path)
            self.streamlit_window_ui.report_pathTextEdit.setText(
                str(self.project_controller.report_path)
            )

        parameters_path = None
        if self.project_controller.report_path is not None:
            parameters_path = str(
                self.project_controller.report_path.joinpath("parameters.yml")
            )
            self.streamlit_window_ui.parameters_ymlTextEdit.setText(parameters_path)

            files = [f for f in os.listdir(analysis_path) if f.endswith(".h5ad")]
            if not files:
                self.handle_missing_h5ad()
            else:
                self.streamlit_window_ui.buttonBox.button(
                    QtWidgets.QDialogButtonBox.StandardButton.Ok
                ).setEnabled(True)

        self.logger.log.debug("Setting analysis path to %s" % analysis_path)
        self.logger.log.debug(
            "Setting report path to %s" % str(self.project_controller.report_path)
        )
        self.logger.log.debug("Setting parameters yml path to %s" % parameters_path)

    def set_connections(self):
        """Set up the connections for the streamlit solution dialog."""
        self.streamlit_window_ui.buttonBox.accepted.connect(
            self._run_streamlit_solution
        )
        self.streamlit_window_ui.buttonBox.rejected.connect(
            self.streamlit_window.reject
        )

        # Connect the buttons to the appropriate methods
        self.streamlit_window_ui.data_pathButton.clicked.connect(self.load_data_path)
        self.streamlit_window_ui.report_pathButton.clicked.connect(
            self.load_report_path
        )
        self.streamlit_window_ui.parameters_ymlButton.clicked.connect(
            self.load_parameters_file
        )

    def _run_streamlit_solution(self):
        solution = "de.mdc:data_viewer:0.1.0"

        if self.interactive:
            data_path = self.streamlit_window_ui.data_pathTextEdit.toPlainText()
            report_path = self.streamlit_window_ui.report_pathTextEdit.toPlainText()
            parameter_yml = str(Path(report_path).joinpath("parameters.yml"))
        else:
            data_path = str(self.project_controller.analysis_path)
            report_path = str(self.project_controller.report_path)
            parameter_yml = str(
                self.project_controller.report_path.joinpath("parameters.yml")
            )

        self.logger.log.info(
            "Running streamlit solution with data path %s and report path %s"
            % (data_path, report_path)
        )

        argv = [
            str(os.path.dirname(os.path.realpath(__file__))),
            "--data_path=%s" % str(data_path),
            "--report_path=%s" % str(report_path),
            "--parameter_yml=%s" % parameter_yml,
            "--run_online=%s" % str(self.interactive),
        ]

        self.logger.log.debug(f"Streamlit solution argv: {argv}")

        self.task = RunSolutionTask(self.album_api, solution, argv)

        # Connect the error signal to the handle_error method
        self.task.on_error = self.handle_error

        # connect the finished signal to the handle_segmentation_result method
        self.task.on_finished = self.handle_streamlit_result

        QThreadPool().globalInstance().start(self.task)

    def handle_streamlit_result(self):
        """Handle the result of the streamlit solution execution."""
        output_file = self.project_controller.project_files_path.joinpath(
            "quantification_done.txt"
        )
        with open(output_file, "w") as f:
            f.write(
                f"Quantification successfully done at {self.project_controller.analysis_path}\n"
            )
            f.write(f"Active Zarr file: {self.project_controller.active_zarr_file}\n")
            f.write(f"Date: {str(datetime.now().strftime('%Y%m%d_%H%M%S'))}")

        self.task = None

    def handle_error(self, e):
        """Handle errors that occur during the execution of the streamlit solution."""
        self.logger.log.error(
            f"An error occurred. See the log for further information: {str(e)}"
        )
        if self.interactive:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Icon.Critical)
            msg.setText("An error occurred")
            msg.setInformativeText(str(e))
            msg.setWindowTitle("Error")
            msg.exec()



"""Controller for managing the opening and creation of projects in ari3d."""
from __future__ import annotations

import glob
import os
import shutil
from datetime import datetime
from pathlib import Path

from PyQt6.QtWidgets import QFileDialog, QMessageBox, QWidget

from ari3d.gui.ari3d_logging import Ari3dLogger
from ari3d.gui.model.decorators import fallback
from ari3d.resources.default_values import DefaultValues
from ari3d.utils.conversion import glob_re, tiffs2zarr
from ari3d.utils.operations.file_operations import (
    create_path_recursively,
    get_dict_from_yml,
    write_dict_to_yml,
)

CURR_PATH = Path(os.path.dirname(os.path.realpath(__file__)))


class OpenProjectController:
    """Controller class for managing the opening and creation of projects in ari3d."""

    def __init__(self, parent: QWidget):
        """Initialize the OpenProjectController."""
        self.logger = Ari3dLogger()
        self.parent = parent
        self.project_path = None
        self.project_files_path = None
        self.gray_path = None
        self.masks_folder = None
        self.report_path = None
        self.analysis_path = None
        self.active_zarr_file = None
        self.active_zarr_file_altered = None
        self.particle_seg_path = None
        self.particle_seg_image_path = None
        self.parameters = None
        self.load_parameters(
            CURR_PATH.parent.parent.parent.joinpath("resources", "parameters.yml")
        )

    def open_project(self, project_path: Path = None, interactive=True):
        """Open a project from the given path."""
        self.logger.log.info("Opening project...")
        self.set_project_path(project_path)

        if not self.project_path:
            return

        # check if zarr file in zarr folder
        zarr_files = glob.glob(str(self.project_files_path.joinpath("*.zarr")))

        # remove "_altered" from zarr files as they are not the original files
        zarr_files = [f for f in zarr_files if "_altered.zarr" not in f]
        self.logger.log.debug("Zarr files: %s" % zarr_files)

        zarr_files_altered = glob.glob(
            str(self.project_files_path) + os.sep + "*_altered.zarr"
        )
        self.logger.log.debug("Zarr files altered: %s" % zarr_files_altered)

        # if no zarr files in folder, assume tiff files present and convert them to zarr to create a new project
        if len(zarr_files) == 0:
            if interactive:
                # message: seems like this is not a project folder, do you want to create a new project?
                QMessageBox.information(
                    self.parent,
                    "No Project Found",
                    "No project found in the given folder.",
                )
            self.project_path = None
            return

        if len(zarr_files) == 1:
            # check whether tiff files in grey folder exist
            tiff_files = list(
                glob_re(
                    r".*\.(tif|tiff|TIF|TIFF)$", os.listdir(path=str(self.gray_path))
                )
            )
            if len(tiff_files) == 0:
                self.logger.log.error(
                    "No tiff files in the grey folder %s that should have"
                    " been created during project creation. Cannot open project."
                    % str(self.project_path)
                )
                self.project_path = None
                return

            self.logger.log.info("Zarr file found: %s" % zarr_files[0])
            self.active_zarr_file = Path(zarr_files[0])
            self.logger.log.info(
                "Set active zarr file to %s" % str(self.active_zarr_file)
            )
        else:
            self.logger.log.error(
                "More than one zarr file in the given folder %s" % self.project_path
            )
            self.project_path = None
            return

        if len(zarr_files_altered) == 1:
            self.active_zarr_file_altered = Path(zarr_files_altered[0])
            self.logger.log.info(
                "Set active zarr file to %s" % str(self.active_zarr_file_altered)
            )
        elif len(zarr_files_altered) > 1:
            self.logger.log.error(
                "More than one altered zarr file in the given folder %s. Cannot open project."
                % str(self.project_path)
            )
            self.project_path = None
            return

        # ensure all necessary folders exist
        create_path_recursively(self.gray_path)
        create_path_recursively(self.project_files_path)
        create_path_recursively(self.report_path)
        create_path_recursively(self.analysis_path)
        create_path_recursively(self.masks_folder)
        create_path_recursively(self.particle_seg_path)
        create_path_recursively(self.particle_seg_image_path)

        # load parameters
        if self.report_path.joinpath("parameters.yml").exists():
            self.logger.log.info(
                "Parameters file found: %s"
                % str(self.report_path.joinpath("parameters.json"))
            )
            self.load_parameters(self.report_path.joinpath("parameters.yml"))
        else:
            # save default parameters file to report folder
            self.logger.log.info(
                "Parameters file not found. Copying default parameters file to report folder..."
            )
            self.save_parameters()

        self.logger.log.info("Project opened successfully!")

    def load_parameters(self, parameters_file: Path):
        """Load parameters from a YAML file."""
        self.logger.log.info("Loading parameters from %s" % str(parameters_file))
        self.parameters = get_dict_from_yml(parameters_file)
        self.logger.log.info("Parameters loaded successfully!")

    def save_parameters(self):
        """Save the current parameters to a YAML file in the report folder."""
        self.logger.log.debug(
            "Saving parameters to %s "
            % str(self.report_path.joinpath("parameters.yml"))
        )
        write_dict_to_yml(
            str(self.report_path.joinpath("parameters.yml")), self.parameters
        )

    @fallback(lambda self: self.fallback_routine())
    def create_project(self, fallback_routine):
        """Create a new project by selecting a directory."""
        project_path = QFileDialog.getExistingDirectory(self.parent, "Select Directory")

        return self._create_project(project_path)

    def _create_project(self, project_path, interactive=True):
        if not project_path:
            return
        self.set_project_path(project_path)

        # check if tiff files exist in the project folder
        images_files = list(
            glob_re(
                r".*\.(tif|tiff|TIF|TIFF)$", os.listdir(path=str(self.project_path))
            )
        )
        if len(images_files) == 0:
            self.logger.log.error(
                "No image files in the given folder %s. Cannot create Project."
                % str(self.project_path)
            )
            self.project_path = None
            return

        # ensure moving tiff files is OK with the user
        if interactive:
            reply = QMessageBox.question(
                self.parent,
                "Confirm Move",
                "Do you want to move TIFF files to the grey folder? Cannot create project without moving files.",
                QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
                QMessageBox.StandardButton.No,
            )
        else:
            reply = QMessageBox.StandardButton.Yes

        if reply == QMessageBox.StandardButton.No:
            self.logger.log.info("User cancelled moving TIFF files.")
            self.project_path = None
            return

        # make sure the folders exist if not create them
        self.logger.log.debug("Creating necessary folders...")
        create_path_recursively(self.gray_path)
        create_path_recursively(self.project_files_path)
        create_path_recursively(self.report_path)
        create_path_recursively(self.analysis_path)
        create_path_recursively(self.masks_folder)
        create_path_recursively(self.particle_seg_path)
        create_path_recursively(self.particle_seg_image_path)
        self.logger.log.info("Converting tiff files to zarr...")
        self.active_zarr_file = Path(
            tiffs2zarr(
                str(self.project_path),
                str(self.project_files_path),
                DefaultValues.n_jobs.value,
            )
        )
        self.logger.log.info("Set active zarr file to %s" % str(self.active_zarr_file))
        assert (
            self.active_zarr_file.exists()
        ), "Zarr file not created. Check logs for errors."
        self.logger.log.info("Moving tiff files to grey folder...")

        for image_file in images_files:
            os.rename(
                str(self.project_path.joinpath(image_file)),
                str(self.gray_path.joinpath(image_file)),
            )

        self.logger.log.info("Project created successfully!")
        self.open_project(self.project_path, interactive)

        # create output file for snakemake
        output_file = self.project_files_path.joinpath("project_created.txt")
        with open(output_file, "w") as f:
            f.write(f"Project created successfully at {self.project_path}\n")
            f.write(f"Active Zarr file: {self.active_zarr_file}\n")
            f.write(f"Date: {str(datetime.now().strftime('%Y%m%d_%H%M%S'))}")

        return

    def fallback_routine(self):
        """Fallback routine to execute when an error occurs."""
        self.logger.log.info("Executing fallback routine...")
        self.project_path = None

        # remove the project folder if it was created
        if self.gray_path is not None and self.gray_path.exists():
            shutil.rmtree(str(self.gray_path))

        if self.project_files_path is not None and self.project_files_path.exists():
            shutil.rmtree(str(self.project_files_path))

        if self.report_path is not None and self.report_path.exists():
            shutil.rmtree(str(self.report_path))

        if self.analysis_path is not None and self.analysis_path.exists():
            shutil.rmtree(str(self.analysis_path))

        if self.masks_folder is not None and self.masks_folder.exists():
            shutil.rmtree(str(self.masks_folder))

    def set_project_path(self, folder_path: Path = None):
        """Set the project path and initialize related paths."""
        self.logger.log.debug("Opening project...")
        if not folder_path:
            folder_path = QFileDialog.getExistingDirectory(
                self.parent, "Select Directory", str(Path.home())
            )

        if not folder_path:
            return  # user cancelled

        if Path(folder_path).stem in DefaultValues.forbidden_project_names.value:
            self.logger.log.error(
                "Project folder cannot be named %s !" % Path(folder_path).stem
            )
            return

        self.logger.log.info("Set project path to %s" % folder_path)
        self.logger.log.info(
            "Set gray path to %s"
            % Path(folder_path).joinpath(DefaultValues.gray_folder_name.value)
        )
        self.logger.log.info(
            "Set project files output path to %s"
            % Path(folder_path).joinpath(DefaultValues.project_files_folder_name.value)
        )
        self.logger.log.info(
            "Set report output path to %s"
            % Path(folder_path).joinpath(DefaultValues.report_folder_name.value)
        )
        self.logger.log.info(
            "Set analysis output path to %s"
            % Path(folder_path).joinpath(DefaultValues.analysis_folder_name.value)
        )
        self.logger.log.debug(
            "Set segmentation output path to %s"
            % Path(folder_path).joinpath(DefaultValues.segmentation_folder_name.value)
        )
        self.logger.log.debug(
            "Set particle seg image path to %s"
            % Path(folder_path).joinpath(
                DefaultValues.segmentation_folder_name.value,
                DefaultValues.segmentation_input_folder_name.value,
            )
        )

        self.project_path = Path(folder_path)
        self.gray_path = Path(self.project_path).joinpath(
            DefaultValues.gray_folder_name.value
        )
        self.analysis_path = Path(self.project_path).joinpath(
            DefaultValues.analysis_folder_name.value
        )
        self.project_files_path = Path(self.project_path).joinpath(
            DefaultValues.project_files_folder_name.value
        )
        self.report_path = Path(self.project_path).joinpath(
            DefaultValues.report_folder_name.value
        )
        self.masks_folder = Path(self.project_path).joinpath(
            DefaultValues.masks_folder_name.value
        )
        self.particle_seg_path = Path(self.project_files_path).joinpath(
            DefaultValues.segmentation_folder_name.value
        )
        self.particle_seg_image_path = Path(self.particle_seg_path).joinpath(
            DefaultValues.segmentation_input_folder_name.value
        )



"""Module for running snakemake workflow for the ARI3D project."""
import argparse
import runpy
import sys
from pathlib import Path

# this file
cur_file = Path(__file__).resolve()


def run_snakemake(args):
    """Run the Snakemake workflow with the provided arguments."""
    sys.argv = [
        "snakemake",
        "all",
        "--snakefile",
        str(cur_file.parent.joinpath("Snakefile")),
        "--cores",
        str(args.cores),
        "--config",
        f"input_dir={args.input_dir}",
        f"parameters_file={args.parameters_file}",
        f"loglevel={args.loglevel}",
    ]

    # Run Snakemake as module
    runpy.run_module("snakemake", run_name="__main__")


def main():
    """Parse arguments and run the Snakemake workflow."""
    parser = argparse.ArgumentParser(
        prog="ari3d", description="ari3d snakemake workflow entry point", add_help=True
    )

    parser.add_argument("--input_dir", help="Input directory containing the data files")
    parser.add_argument("--parameters_file", help="Path to the parameters file")
    parser.add_argument(
        "--cores",
        type=int,
        help="Number of cores to use for the workflow",
        default=1,
        required=False,
    )
    parser.add_argument(
        "--loglevel", type=str, help="Logging level", default="INFO", required=False
    )

    # Show help if no arguments are passed
    if len(sys.argv) == 1:
        parser.print_help(sys.stderr)
        sys.exit(1)

    args = parser.parse_args()

    run_snakemake(args)


if __name__ == "__main__":
    main()



import gc
import os
import re
from tkinter.filedialog import askdirectory

import anndata
import nibabel as nib
import numpy as np
import pandas as pd
from skimage import io, measure


def glob_re(pattern, strings):
    """Regular expression matching."""
    return filter(re.compile(pattern).match, strings)


def _load_images(binary_image_path, start_slice, end_slice):
    """Load the images. Supported formate is TIFF and NIFTI."""
    tiff_files = sorted(
        list(
            glob_re(
                r".*\.(tif|tiff|TIF|TIFF)$", os.listdir(path=str(binary_image_path))
            )
        )
    )
    tiff_files = [os.path.join(binary_image_path, f) for f in tiff_files]

    nifti_files = sorted(
        list(glob_re(r".*\.(nii|nii.gz)$", os.listdir(path=str(binary_image_path))))
    )
    nifti_files = [os.path.join(binary_image_path, f) for f in nifti_files]

    if len(tiff_files) > 1:
        # Multiple TIFF files - treat as 2D stack
        cv_img = []
        for img in tiff_files:
            n = io.imread(img, as_gray=True)
            cv_img.append(n)
        images = np.dstack(cv_img)
        images = np.rollaxis(images, -1)
        del cv_img
        del n
    elif len(tiff_files) == 1:
        # Single TIFF file - treat as 3D image
        images = io.imread(tiff_files[0])
    elif len(nifti_files) == 1:
        # Single NIfTI file - treat as 3D image
        images = nib.load(nifti_files[0]).get_fdata()
    else:
        raise ValueError("No compatible image files found in the specified directory.")

    # slicing
    if start_slice is not None and end_slice is not None:
        if start_slice == end_slice:
            raise ValueError("Start and end slice cannot be the same.")
        if images.ndim == 3:
            if start_slice >= 0 and end_slice < images.shape[0]:
                images = images[start_slice : end_slice + 1, :, :]
            else:
                raise ValueError("Invalid slice range for the 3D image stack.")
        else:
            raise ValueError("Slice range can only be specified for 3D image stacks.")

    # Calculate the maximum grey value
    max_grey_value = np.max(images)

    # Decide which unit type image should be based on the max grey value
    if max_grey_value <= 255:
        # Convert to uint8
        images = images.astype(np.uint8)
    elif max_grey_value <= 65535:
        # Convert to uint16
        images = images.astype(np.uint16)
    else:
        images = images.astype(np.uint32)

    gc.collect()
    return images


def load_and_process_labelled_image(binary_image_path, start_slice, end_slice):
    """Load and process labelled images."""
    labels = _load_images(binary_image_path, start_slice, end_slice)
    if (
        len(np.unique(labels)) < 3
    ):  # if mask was exported from Dragonfly the particles have greyvalue 255, from Avizo or ImageJ change to 1.
        labels = measure.label(labels)
    else:
        labels = labels
    binary = np.where(labels > 1, 1, 0)
    binary = binary.astype(np.uint8)
    print("Number of particles before processing", labels.max())
    print("Image size", labels.shape)
    return labels, binary


def load_non_binary_images(non_binary_image_path, start_slice, end_slice):
    """Load gray-scale images."""
    non_binary = _load_images(non_binary_image_path, start_slice, end_slice)
    non_binary = non_binary.astype(np.uint16)
    return non_binary


def directory():
    """Ask for a directory."""
    # prompt to choose the directory. Must have folders with names 'binary','grey','data'
    path = askdirectory(title="select folder")
    return path


def convert_h5ad(histograms_df, path_to_save_histograms):
    """Convert histograms to h5ad and save."""
    histograms_df.index = histograms_df.index.astype(str)
    histograms_df.columns = histograms_df.columns.astype(str)
    histograms_df.columns = ["bin_" + str(col) for col in histograms_df.columns]

    obs = pd.DataFrame(index=histograms_df.index)
    var = pd.DataFrame(index=histograms_df.columns)

    adata = anndata.AnnData(X=histograms_df, obs=obs, var=var)
    adata.write(path_to_save_histograms)

    return histograms_df
